@@ -31,7 +31,7 @@ namespace {
                         unsigned& pline, string::size_type& ppos) {
     while ( pline < size ) {
       string::size_type wbeg = preStmt[pline].find_first_not_of(" \t", ppos);
-      if ( preStmt[pline][wbeg] == '\\' || wbeg == string::npos ) {
+      if ( wbeg == string::npos || preStmt[pline][wbeg] == '\\' ) {
         ++pline;
         if ( pline < size ) { ppos = 0; } else { return ""; }
       } else {
@@ -147,6 +147,10 @@ void process_c_or_cxx(istream& is, const char* infile, ostream& os,
     /* workaround for bogus getline implementations */
     if ( line.size() == 1 && line[0] == '\0' ) break;
 
+    /* remove possible trailing Carriage Return from line */
+    if ( line.size() > 0 && line[line.length()-1] == '\r' )
+      line.erase(line.length()-1);
+
     if ( preContLine ) {
       /*
        * preprocessor directive continuation
