@@ -2,7 +2,7 @@
  * VampirTrace
  * http://www.tu-dresden.de/zih/vampirtrace
  *
- * Copyright (c) 2005-2008, ZIH, TU Dresden, Federal Republic of Germany
+ * Copyright (c) 2005-2012, ZIH, TU Dresden, Federal Republic of Germany
  *
  * Copyright (c) 1998-2005, Forschungszentrum Juelich, Juelich Supercomputing
  *                          Centre, Federal Republic of Germany
@@ -71,8 +71,8 @@ static char* vt_exec = NULL;
 /* platform specific initialization */
 void vt_pform_init() {
   int  pid = getpid();
-  char exec_proc[512];
-  char exec[1024];
+  char exec_proc[VT_PATH_MAX];
+  char exec[VT_PATH_MAX];
   int  exec_len;
 
 #if TIMER == TIMER_CYCLE_COUNTER
@@ -88,7 +88,7 @@ void vt_pform_init() {
       strtok(line, ":");
 
       vt_ticks_per_sec =
-	strtol((char*) strtok(NULL, " \n"), (char**) NULL, 0) * 1e6;
+	strtol((char*) strtok(NULL, " \n"), (char**) NULL, 0) * 1000000LL;
     }
     else if (!strncmp("timebase", line, 8))
     {
@@ -146,7 +146,7 @@ void vt_pform_init() {
 
   /* get full path of executable */
   snprintf(exec_proc, sizeof (exec_proc), VT_PROCDIR"%d/exe", pid);
-  exec_len = readlink(exec_proc, exec, sizeof (exec));
+  exec_len = readlink(exec_proc, exec, sizeof (exec)-1);
   if(exec_len != -1)
   {
     exec[exec_len] = '\0';
@@ -161,10 +161,10 @@ char* vt_pform_gdir() {
 
 /* directory of local file system  */
 char* vt_pform_ldir() {
-#ifdef PFORM_LDIR
-  return PFORM_LDIR;
+#ifdef DEFAULT_PFORM_LDIR
+  return DEFAULT_PFORM_LDIR;
 #else
-  return "/tmp";
+  return ".";
 #endif
 }
 
@@ -177,17 +177,17 @@ char* vt_pform_exec()
 /* clock resolution */
 uint64_t vt_pform_clockres() {
 #if TIMER == TIMER_DCLOCK
-  return 1e15;
+  return 1000000000000000LL;
 #elif TIMER == TIMER_CYCLE_COUNTER
   return vt_ticks_per_sec;
 #elif TIMER == TIMER_CLOCK_GETTIME
-  return 1e9;
+  return 1000000000LL;
 #elif TIMER == TIMER_GETTIMEOFDAY
-  return 1e6;
+  return 1000000LL;
 #elif TIMER == TIMER_PAPI_REAL_CYC
   return vt_metric_clckrt();
 #elif TIMER == TIMER_PAPI_REAL_USEC
-  return 1e6;
+  return 1000000LL;
 #endif
 }
 
@@ -205,11 +205,11 @@ uint64_t vt_pform_wtime() {
 #elif TIMER == TIMER_CLOCK_GETTIME
   struct timespec tp;
   clock_gettime(CLOCK_REALTIME, &tp);
-  return ((tp.tv_sec - vt_time_base) * 1e9) + tp.tv_nsec;
+  return ((tp.tv_sec - vt_time_base) * 1000000000LL) + tp.tv_nsec;
 #elif TIMER == TIMER_GETTIMEOFDAY
   struct timeval tp;
   gettimeofday(&tp, 0);
-  return ((tp.tv_sec - vt_time_base) * 1e6) + tp.tv_usec;
+  return ((tp.tv_sec - vt_time_base) * 1000000LL) + tp.tv_usec;
 #elif TIMER == TIMER_PAPI_REAL_CYC
   return vt_metric_real_cyc();
 #elif TIMER == TIMER_PAPI_REAL_USEC
