@@ -25,10 +25,8 @@
 #include "ompi_config.h"
 
 #include "opal/class/opal_list.h"
-#include "ompi/class/ompi_rb_tree.h"
 #include "opal/mca/mca.h"
 #include "ompi/mca/mpool/mpool.h"
-#include "opal/threads/mutex.h" 
 
 BEGIN_C_DECLS
 
