@@ -10,7 +10,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2006-2007 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2007      Los Alamos National Security, LLC.  All rights
+ * Copyright (c) 2007-2012 Los Alamos National Security, LLC.  All rights
  *                         reserved. 
  * Copyright (c) 2008-2009 Sun Microsystems, Inc.  All rights reserved.
  * Copyright (c) 2011      IBM Corporation.  All rights reserved.
@@ -618,12 +618,14 @@ static int setup_launch(int *argcptr, char ***argvptr,
             asprintf (&final_cmd,
                       "%s%s%s PATH=%s/%s:$PATH ; export PATH ; "
                       "LD_LIBRARY_PATH=%s/%s:$LD_LIBRARY_PATH ; export LD_LIBRARY_PATH ; "
+                      "DYLD_LIBRARY_PATH=%s/%s:$DYLD_LIBRARY_PATH ; export DYLD_LIBRARY_PATH ; "
                       "%s %s",
                       (opal_prefix != NULL ? "OPAL_PREFIX=" : " "),
                       (opal_prefix != NULL ? opal_prefix : " "),
                       (opal_prefix != NULL ? " ; export OPAL_PREFIX;" : " "),
                       prefix_dir, bin_base,
                       prefix_dir, lib_base,
+                      prefix_dir, lib_base,
                       (orted_prefix != NULL ? orted_prefix : " "),
                       (full_orted_cmd != NULL ? full_orted_cmd : " "));
         } else if (ORTE_PLM_RSH_SHELL_TCSH == remote_shell ||
@@ -648,6 +650,12 @@ static int setup_launch(int *argcptr, char ***argvptr,
                       "setenv LD_LIBRARY_PATH %s/%s ; "
                       "if ( $?OMPI_have_llp == 1 ) "
                       "setenv LD_LIBRARY_PATH %s/%s:$LD_LIBRARY_PATH ; "
+                      "if ( $?DYLD_LIBRARY_PATH == 1 ) "
+                      "set OMPI_have_dllp ; "
+                      "if ( $?DYLD_LIBRARY_PATH == 0 ) "
+                      "setenv DYLD_LIBRARY_PATH %s/%s ; "
+                      "if ( $?OMPI_have_dllp == 1 ) "
+                      "setenv DYLD_LIBRARY_PATH %s/%s:$DYLD_LIBRARY_PATH ; "
                       "%s %s",
                       (opal_prefix != NULL ? "setenv OPAL_PREFIX " : " "),
                       (opal_prefix != NULL ? opal_prefix : " "),
@@ -655,6 +663,8 @@ static int setup_launch(int *argcptr, char ***argvptr,
                       prefix_dir, bin_base,
                       prefix_dir, lib_base,
                       prefix_dir, lib_base,
+                      prefix_dir, lib_base,
+                      prefix_dir, lib_base,
                       (orted_prefix != NULL ? orted_prefix : " "),
                       (full_orted_cmd != NULL ? full_orted_cmd : " "));
         } else {
@@ -998,7 +1008,7 @@ plm_rsh_report_orted_launch(int status, orte_process_name_t* sender,
     orte_process_name_t peer;
     char *rml_uri = NULL;
     int rc, idx;
-    orte_proc_t *daemon;
+    orte_proc_t *daemon=NULL;
     orte_job_t *jdatorted;
     
     orted_failed_launch = true;
@@ -1062,7 +1072,8 @@ CLEANUP:
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                          orted_failed_launch ? "failed" : "completed",
                          ORTE_NAME_PRINT(&peer),
-                         ORTE_NAME_PRINT(sender), daemon->rml_uri));
+                         ORTE_NAME_PRINT(sender),
+                         (NULL == daemon) ? "UNKNOWN" : daemon->rml_uri));
 
     if (orted_failed_launch) {
         if( NULL != rml_uri ) free(rml_uri);
@@ -1114,6 +1125,12 @@ int orte_plm_rsh_launch(orte_job_t *jdata)
 	goto launch_apps;
     }
 
+    if (jdata->controls & ORTE_JOB_CONTROL_DEBUGGER_DAEMON) {
+        /* debugger daemons */
+        active_job = jdata->jobid;
+        goto launch_apps;
+    }
+
     if (jdata->controls & ORTE_JOB_CONTROL_LOCAL_SLAVE) {
         /* if this is a request to launch a local slave,
          * then we will not be launching an orted - we will
